#!/usr/bin/python

import glob
import lxml.html
import os
import os.path
import StringIO
import urllib
import zipfile

OUTPUT_FILE = 'opm.epub'
BOOK_TITLE = "Louis D. Brandeis - Other people's money"

destdir = lambda x: os.path.join(os.path.expanduser('ebook/'), x)

toc_ncx = """<?xml version="1.0" encoding="UTF-8"?>
<ncx xmlns="http://www.daisy.org/z3986/2005/ncx/" version="2005-1">

<head>
    <meta name="dtb:uid" content="http://www.law.louisville.edu/library/collections/brandeis/node/191"/>
    <meta name="dtb:depth" content="1"/>
    <meta name="dtb:totalPageCount" content="0"/>
    <meta name="dtb:maxPageNumber" content="0"/>
</head>

<docTitle>
    <text>%s</text>
</docTitle>

<navMap>
    %s
</navMap>
</ncx>"""

chapter_template = '''<?xml version='1.0' encoding='utf-8'?>\n
<html xmlns="http://www.w3.org/1999/xhtml"><head>\n
<title>%(title)s</title>\n
</head><body>\n
%(data)s
</body></html>
'''

container_xml = '''<?xml version="1.0"?>
	<container version="1.0" xmlns="urn:oasis:names:tc:opendocument:xmlns:container">
	<rootfiles>
		<rootfile full-path="OEBPS/Content.opf" media-type="application/oebps-package+xml"/>
	</rootfiles>
</container>'''

index_tpl = '''<package version="2.0"
	xmlns="http://www.idpf.org/2007/opf" unique-identifier="BookID">
	<metadata xmlns:dc="http://purl.org/dc/elements/1.1/" xmlns:opf="http://www.idpf.org/2007/opf">
		%(metadata)s
	</metadata>
	<manifest>
		<item id="ncx" href="toc.ncx" media-type="application/x-dtbncx+xml" />
		%(manifest)s
	</manifest>
	<spine toc="ncx">
		%(spine)s
	</spine>
</package>'''



def get_url(url, raw=False):
	print 'Prenasam', url
	u = urllib.urlopen(url)
	resp = u.read()
	return [resp]

def make_tree(url, data, encoding=None):
	"builds lxml tree"
	s = StringIO.StringIO(data)
	parser = lxml.html.HTMLParser(encoding=encoding)
	doc = lxml.html.parse(s, parser=parser)
	html = doc.getroot()
	html.make_links_absolute(url)
	return html

def write_chapter(number, data, title):
	fd = open(destdir('chap%.2d.xhtml' % number), 'wb')
	fd.write(chapter_template % {
		'title': title,
		'data': data
		})
	fd.close()

def write_epub():
	print 'Zapisujem EPUB'
	epub = zipfile.ZipFile(OUTPUT_FILE, 'w')

	# The first file must be named "mimetype"
	epub.writestr("mimetype", "application/epub+zip")

	# The filenames of the HTML are listed in html_files
	html_files = list(sorted(glob.glob(destdir('') + '*.xhtml')))
	image_files = list(sorted(glob.glob(destdir('images') + '/*')))

	# We need an index file, that lists all other HTML files
	# This index file itself is referenced in the META_INF/container.xml
	# file
	epub.writestr("META-INF/container.xml", container_xml);

	# The index file is another XML file, living per convention
	# in OEBPS/Content.xml

	manifest = ""
	spine = ""
	ncx = ""

	# Write each HTML file to the ebook, collect information for the index
	for i, html in enumerate(html_files):
		#print html
		basename = os.path.basename(html)
		manifest += '<item id="file_%s" href="%s" media-type="application/xhtml+xml"/>' % (
									i+1, basename)
		spine += '<itemref idref="file_%s" />' % (i+1)
		ncx += """<navPoint id="file_%s" playOrder="%s">
	<navLabel>
		<text>Chapter 1</text>
	</navLabel>
	<content src="chap%.2d.xhtml"/>
</navPoint>
""" % (i+1, i+1, i)
		
		epub.write(html, 'OEBPS/'+basename)

	for n, img in enumerate(image_files):
		#print img
		basename = os.path.basename(img)
		manifest += '''<item id="%s" href="images/%s"
								media-type="image/jpeg" />''' % (basename, basename)
		epub.write(img, 'OEBPS/images/' + basename)
	
	metadata = "\n".join([
		"<dc:title>Other people's money</dc:title>",
		'<dc:creator opf:role="aut">Louis D. Brandeis</dc:creator>',
		"<dc:language>en-US</dc:language>",
		"<dc:rights>exclusive property of the University of Louisville</dc:rights>",
		"<dc:publisher>University of Louisville, http://www.law.louisville.edu/library/collections/brandeis/node/191</dc:publisher> ",
		"<dc:identifier id=\"BookID\">urn:isbn:978-0312103149</dc:identifier>",
		])
	
	# Finally, write the index
	epub.writestr('OEBPS/Content.opf', index_tpl % {
			'manifest': manifest,
			'spine': spine,
			'metadata': metadata,
	})
	
	epub.writestr('OEBPS/toc.ncx', toc_ncx % (BOOK_TITLE, ncx))


def get_images(doc):
	
	for i in doc.xpath('.//img'):
		iurl = i.attrib['src']
		data = get_url(iurl, raw=True)
		imgname = iurl.rsplit('/', 1)[1]
		if not os.path.exists(destdir(os.path.join('images'))):
			os.makedirs(destdir(os.path.join('images')))
		fd = open(destdir(os.path.join('images', imgname)), 'wb')
		fd.write(data[0])
		fd.close()
		i.attrib['src'] = 'images/' + imgname
	

def main():
	if not os.path.exists('ebook'):
		# need to download
		url = "http://www.law.louisville.edu/library/collections/brandeis/node/191"
		
		doc = make_tree(data=get_url(url)[0], url=url)
		#print get_url(url)[0]
		#print doc
		
		div = doc.xpath('//div[@id="squeeze"]')[0]
		get_images(div)
		
		final = []
		
		urls = [i.attrib['href'] for i in div.xpath('.//a')]
		
		for n, u in enumerate(urls):
			#print u
			doc2 = make_tree(data=get_url(u)[0], url=u)
			
			for a in doc2.xpath('//*[@align]'):
				del a.attrib['align']
			
			txt = doc2.xpath('//div[@id="squeeze"]')[0]
			txt.remove(txt.xpath('./h1')[0])
			for t in txt.xpath('.//h4'):
				t.getparent().remove(t)
			
			txt = txt.xpath('.//div[@class="content"]')[0]
			
			txt.attrib['id'] = 'ch%s' % (n+1)
			a = div.xpath('//a[@href="%s"]' % u)[0]
			a.attrib['href'] = 'chap%.2d.xhtml' % (n+1)
			
			title = txt.xpath('./h1')[0]
			
			get_images(txt)
			write_chapter(n+1, lxml.html.tostring(txt, method='xml'), title)
		
		write_chapter(0, lxml.html.tostring(div, method='xml'), title=BOOK_TITLE)
	
	write_epub()
	

if __name__ == "__main__":
	main()
